package com.example.bdkipoc;

import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class TransactionActivity extends AppCompatActivity {
    private RecyclerView recyclerView;
    private TransactionAdapter adapter;
    private ProgressBar progressBar;
    private FloatingActionButton refreshButton;
    private int page = 0;
    private final int limit = 10;
    private boolean isLoading = false;
    private boolean isLastPage = false;
    private List<Transaction> transactionList = new ArrayList<>();

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_transaction);

        // Set up the toolbar as the action bar
        androidx.appcompat.widget.Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        
        // Enable the back button in the action bar
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
        }

        recyclerView = findViewById(R.id.recyclerView);
        progressBar = findViewById(R.id.progressBar);
        refreshButton = findViewById(R.id.refreshButton);

        adapter = new TransactionAdapter(transactionList);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setAdapter(adapter);

        refreshButton.setOnClickListener(v -> refreshTransactions());

        recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                if (!recyclerView.canScrollVertically(1) && !isLoading && !isLastPage) {
                    loadTransactions(page + 1);
                }
            }
        });

        loadTransactions(0);
    }

    private void refreshTransactions() {
        page = 0;
        isLastPage = false;
        transactionList.clear();
        adapter.notifyDataSetChanged();
        loadTransactions(0);
    }

    private void loadTransactions(int pageToLoad) {
        isLoading = true;
        progressBar.setVisibility(View.VISIBLE);
        new FetchTransactionsTask(pageToLoad).execute();
    }

    private class FetchTransactionsTask extends AsyncTask<Void, Void, List<Transaction>> {
        private int pageToLoad;
        private boolean error = false;
        private int total = 0;

        FetchTransactionsTask(int page) {
            this.pageToLoad = page;
        }

        @Override
        protected List<Transaction> doInBackground(Void... voids) {
            List<Transaction> result = new ArrayList<>();
            try {
                String urlString = "https://be-edc.msvc.app/transactions?page=" + pageToLoad + "&limit=" + limit + "&sortOrder=DESC&from_date=&to_date=&location_id=0&merchant_id=0&tid=73001500&mid=71000026521&sortColumn=id";
                URI uri = new URI(urlString);
                URL url = uri.toURL();
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("accept", "*/*");
                int responseCode = conn.getResponseCode();
                if (responseCode == 200) {
                    BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                    StringBuilder response = new StringBuilder();
                    String line;
                    while ((line = in.readLine()) != null) {
                        response.append(line);
                    }
                    in.close();
                    JSONObject jsonObject = new JSONObject(response.toString());
                    JSONObject results = jsonObject.getJSONObject("results");
                    total = results.getInt("total");
                    JSONArray data = results.getJSONArray("data");
                    for (int i = 0; i < data.length(); i++) {
                        JSONObject t = data.getJSONObject(i);
                        Transaction tx = new Transaction(
                                t.getInt("id"),
                                t.getString("type"),
                                t.getString("channel_category"),
                                t.getString("channel_code"),
                                t.getString("reference_id"),
                                t.getString("amount"),
                                t.getString("cashflow"),
                                t.getString("status"),
                                t.getString("created_at"),
                                t.getString("merchant_name")
                        );
                        result.add(tx);
                    }
                } else {
                    error = true;
                }
            } catch (IOException | JSONException | URISyntaxException e) {
                error = true;
            }
            return result;
        }

        @Override
        protected void onPostExecute(List<Transaction> transactions) {
            isLoading = false;
            progressBar.setVisibility(View.GONE);
            if (error) {
                Toast.makeText(TransactionActivity.this, "Failed to fetch transactions", Toast.LENGTH_SHORT).show();
                return;
            }
            if (pageToLoad == 0) {
                transactionList.clear();
            }
            transactionList.addAll(transactions);
            adapter.notifyDataSetChanged();
            page = pageToLoad;
            if (transactionList.size() >= total) {
                isLastPage = true;
            }
        }
    }

    @Override
    public boolean onOptionsItemSelected(android.view.MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            // Handle the back button click
            finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    static class Transaction {
        int id;
        String type;
        String channelCategory;
        String channelCode;
        String referenceId;
        String amount;
        String cashflow;
        String status;
        String createdAt;
        String merchantName;

        Transaction(int id, String type, String channelCategory, String channelCode, String referenceId, String amount, String cashflow, String status, String createdAt, String merchantName) {
            this.id = id;
            this.type = type;
            this.channelCategory = channelCategory;
            this.channelCode = channelCode;
            this.referenceId = referenceId;
            this.amount = amount;
            this.cashflow = cashflow;
            this.status = status;
            this.createdAt = createdAt;
            this.merchantName = merchantName;
        }
    }
}
